# encoding: utf-8
import json

from optionaldict import optionaldict

from .base import CecibsBaseAPI


class Open(CecibsBaseAPI):
    """
    在线课程学课/考试接口
    """
    def lesson_list_open(self, trainplan_id=None, res_type=None):
        """
        课程清单接口

        :param trainplan_id: 计划id
        :param res_type: 默认返回全部类型资源，支持传入指定类型资源：lesson:在线课程;paper:试卷;document:拓展阅读
        :return:
        """
        assert res_type in (None, "lesson", "paper", "document")
        return self._post(
            '/v1/open/lesson_list_open',
            optionaldict({
                'company_code': self.company_code,
                'trainplan_id': trainplan_id,
                'res_type': res_type
            })
        )

    def _gen_page(
        self,
        uri,
        _from,
        trainplan_id,
        resources_id,
        login_name,
        hide_top_bar=0,
        back_url=None,
        show_full_screen_btn=0,
        show_speed_btn=0,
        disable_drag=0
    ):
        """
        生成页面

        :param uri: 接口uri
        :param _from: 平台类型，1：手机端; 2:PC端
        :param trainplan_id: 计划ID
        :param resources_id: 课程ID
        :param login_name: 当前登录的用户名称
        :param hide_top_bar: 是否隐藏顶部菜单栏
        :param back_url: 返回按钮的链接地址
        :param show_full_screen_btn: h5学课是否显示全屏按钮
        :param show_speed_btn: 是否显示倍速
        :param disable_drag: 是否禁止进度条拖拽
        :return: url
        """
        assert _from in (1, 2)
        params = optionaldict({
            "from": _from,
            "code": self.company_code,
            "trainplan_id": trainplan_id,
            "resources_id": resources_id,
            "login_name": login_name,
            "hide_top_bar": 1 if hide_top_bar else 0,
            "back_url": back_url,
            "show_full_screen_btn": 1 if show_full_screen_btn else 0,
            "show_speed_btn": 1 if show_speed_btn else 0,
            "disable_drag": 1 if disable_drag else 0,
        })
        return self._gen_url(uri, params)

    def lesson_open(
            self,
            _from,
            trainplan_id,
            resources_id,
            login_name,
            hide_top_bar=0,
            back_url=None,
            show_full_screen_btn=0,
            show_speed_btn=0,
            disable_drag=0
    ):
        """
        学课/考试页面接口

        :param _from: 平台类型，1：手机端; 2:PC端
        :param trainplan_id: 计划ID
        :param resources_id: 课程ID
        :param login_name: 当前登录的用户名称
        :param hide_top_bar: 是否隐藏顶部菜单栏
        :param back_url: 返回按钮的链接地址
        :param show_full_screen_btn: h5学课是否显示全屏按钮
        :param show_speed_btn: 是否显示倍速
        :param disable_drag: 是否禁止进度条拖拽
        :return: url
        """
        return self._gen_page(
            '/v1/open/lesson_open',
            _from=_from,
            trainplan_id=trainplan_id,
            resources_id=resources_id,
            login_name=login_name,
            hide_top_bar=hide_top_bar,
            back_url=back_url,
            show_full_screen_btn=show_full_screen_btn,
            show_speed_btn=show_speed_btn,
            disable_drag=disable_drag,
        )


    def paper_history(
            self,
            _from,
            trainplan_id,
            resources_id,
            login_name,
            hide_top_bar=False,
            back_url=None,
            show_full_screen_btn=True,
            show_speed_btn=False,
            disable_drag=False
    ):
        """
        考试历史页面接口

        :param _from: 平台类型，1：手机端; 2:PC端
        :param trainplan_id: 计划ID
        :param resources_id: 课程ID
        :param login_name: 当前登录的用户名称
        :param hide_top_bar: 是否隐藏顶部菜单栏
        :param back_url: 返回按钮的链接地址
        :param show_full_screen_btn: h5学课是否显示全屏按钮
        :param show_speed_btn: 是否显示倍速
        :param disable_drag: 是否禁止进度条拖拽
        :return: url
        """

        return self._gen_page(
            '/v1/open/paper_history',
            _from=_from,
            trainplan_id=trainplan_id,
            resources_id=resources_id,
            login_name=login_name,
            hide_top_bar=hide_top_bar,
            back_url=back_url,
            show_full_screen_btn=show_full_screen_btn,
            show_speed_btn=show_speed_btn,
            disable_drag=disable_drag,
        )

    def learn_rate_open(self, _from, trainplan_id, resources_id, login_name):
        """
        获取学习进度接口

        :param _from: 平台类型，1：手机端; 2:PC端
        :param trainplan_id: 计划ID
        :param resources_id: 课程ID
        :param login_name: 当前登录的用户名称
        """
        assert _from in (1, 2)
        self._post('/v1/open/learn_rate_open', {
            'from': _from,
            'company_code': self.company_code,
            'trainplan_id': trainplan_id,
            'resources_id': resources_id,
            'login_name': login_name
        })

    def learn_rate_open_batch(self, _from, data=()):
        """
        批量获取学习进度接口

        :param _from: 平台类型，1：手机端; 2:PC端
        :param data: 请求数据 格式：```[
                                        ("计划id", "课程id", "当前登录的用户名称"),
                                        {
                                            "trainplan_id": "计划id",
                                            "resources_id": "课程id",
                                            "login_name": "当前登录的用户名称"
                                        }
                                   ]```
        """
        assert _from in (1, 2)
        _data = list()
        for d in data:
            sub_data = dict()
            if isinstance(d, (list, tuple)):
                assert len(d) == 3
                sub_data["trainplan_id"], sub_data["resources_id"], sub_data["login_name"] = d
            elif isinstance(d, dict):
                sub_data["trainplan_id"] = d["trainplan_id"]
                sub_data["resources_id"] = d["resources_id"]
                sub_data["login_name"] = d["login_name"]
            else:
                raise TypeError("data type error " + d)
            _data.append(sub_data)

        self._post('/v1/open/learn_rate_open_batch', {
            'from': _from,
            'company_code': self.company_code,
            'data': json.dumps(_data),
        })

    def paper_rate_open_batch(self, _from, data=()):
        """
        批量获取用户历史考试接口

        :param _from: 平台类型，1：手机端; 2:PC端
        :param data: 请求数据 格式：[("计划id", "试卷ID"), {"trainplan_id": "计划id", "resources_id": "试卷ID"}]
        """
        assert _from in (1, 2)
        _data = list()
        for d in data:
            sub_data = dict()
            if isinstance(d, (list, tuple)):
                assert len(d) == 2
                sub_data["trainplan_id"], sub_data["resources_id"] = d
            elif isinstance(d, dict):
                sub_data["trainplan_id"] = d["trainplan_id"]
                sub_data["resources_id"] = d["resources_id"]
            else:
                raise TypeError("data type error " + d)
            _data.append(sub_data)

        self._post('/v1/open/paper_rate_open_batch', {
            'from': _from,
            'company_code': self.company_code,
            'data': json.dumps(_data),
        })



# encoding: utf-8

import logging

from . import api
from .base import BaseClient
from ..storage.cache import EceibsCache

logger = logging.getLogger(__name__)


class EceibsClient(BaseClient):
    auth = api.Auth()
    open = api.Open()
    column = api.Column()

    def __init__(self, company_code, open_secret, storage=None, timeout=None):
        super().__init__(storage=storage, timeout=timeout)
        self.company_code = company_code
        self.open_secret = open_secret
        self.cache = EceibsCache(self.storage)

    def _handle_pre_request(self, method, uri, kwargs):
        if kwargs is None:
            kwargs = dict()
        kwargs.setdefault("headers", dict())
        if 'Authorization' in kwargs['headers']:
            raise ValueError("headers中不允许有 Authorization: " + kwargs['headers']['Authorization'])
        kwargs['headers']['Authorization'] = "Bearer " + self.access_token
        return method, uri, kwargs

    def _handle_request_except(self, e, func, *args, **kwargs):
        # if e.errcode in (33001, 40001, 42001, 40014):
        #     self.cache.access_token.delete()
        #     if self.auto_retry:
        #         return func(*args, **kwargs)

        raise e

    @property
    def access_token(self):
        self.cache.access_token.get()
        token = self.cache.access_token.get()
        if token is None:
            token = self.get_access_token()
            self.cache.access_token.set(value=token, ttl=120)
        return token

    def get_access_token(self):
        return self._request(
            'POST',
            '/v1/auth/login',
            data={'code': self.company_code, 'open_secret': self.open_secret},
            result_processor=lambda x: x['access_token']
        )


class TestEceibsClient(EceibsClient):
    def __init__(self, company_code="apidemo", open_secret=None, storage=None, timeout=None):
        super().__init__(company_code=company_code, open_secret=open_secret, storage=storage, timeout=timeout)

    def get_access_token(self):
        return self._request(
            'GET',
            'http://samples.eceibs.com.cn/apidemo/api/auth.php',
            result_processor=lambda x: x['access_token']
        )



# encoding: utf-8


import json

from ..core.utils import to_text
from . import BaseStorage


class KvStorage(BaseStorage):

    def __init__(self, kvdb, prefix='eceibs'):
        for method_name in ('get', 'set', 'delete'):
            assert hasattr(kvdb, method_name)
        self.kvdb = kvdb
        self.prefix = prefix

    def key_name(self, key):
        return '{0}:{1}'.format(self.prefix, key)

    def get(self, key, default=None):
        key = self.key_name(key)
        value = self.kvdb.get(key)
        if value is None:
            return default
        return json.loads(to_text(value))

    def set(self, key, value, ttl=None):
        if value is None:
            return
        key = self.key_name(key)
        value = json.dumps(value)
        self.kvdb.set(key, value, ttl)

    def delete(self, key):
        key = self.key_name(key)
        self.kvdb.delete(key)



# encoding: utf-8

from .base import CecibsBaseAPI


class Column(CecibsBaseAPI):
    """
    专栏－视频接口
    """
    def column_list(self, _type=1):
        """
        专栏列表接口

        :param _type: 类型 (1:视频)
        """
        return self._post(
            '/v1/column/list', {'type': _type, 'company_code': self.company_code}, result_processor=lambda x: x['list']
        )

    def file_media(self, _id, login_name):
        """
        视频播放－获取m3u8接口

        :param _id: 视频文件id
        :param login_name: 当前登陆的用户名称
        :return: m3u8 url
        """
        return self._gen_url(
            '/v1/file/media',
            {'company_code': self.company_code, 'id': _id, 'login_name': login_name},
            token_key='token'
        )



# encoding: utf-8

import inspect

from . import BaseStorage


def _is_cache_item(obj):
    return isinstance(obj, EceibsCacheItem)


class EceibsCacheItem(object):

    def __init__(self, cache=None, name=None):
        self.cache = cache
        self.name = name

    def key_name(self, key):
        if isinstance(key, (tuple, list)):
            key = ':'.join(key)

        k = '{0}:{1}'.format(self.cache.prefix, self.name)
        if key is not None:
            k = '{0}:{1}'.format(k, key)
        return k

    def get(self, key=None, default=None):
        return self.cache.storage.get(self.key_name(key), default)

    def set(self, key=None, value=None, ttl=None):
        return self.cache.storage.set(self.key_name(key), value, ttl)

    def delete(self, key=None):
        return self.cache.storage.delete(self.key_name(key))


class BaseCache(object):

    def __new__(cls, *args, **kwargs):
        self = super(BaseCache, cls).__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_cache_item)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self, name)
            setattr(self, name, api)
        return self

    def __init__(self, storage, prefix='client'):
        assert isinstance(storage, BaseStorage)
        self.storage = storage
        self.prefix = prefix


class EceibsCache(BaseCache):
    access_token = EceibsCacheItem()



# encoding: utf-8


class CecibsBaseAPI(object):

    API_BASE_URL = None

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, params=None, **kwargs):
        if self.API_BASE_URL and 'api_base_url' not in kwargs:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.get(url, params, **kwargs)

    def _post(self, url, data=None, params=None, **kwargs):
        if self.API_BASE_URL and 'api_base_url' not in kwargs:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(url, data, params, **kwargs)

    def _gen_url(self, url, params=None, **kwargs):
        if self.API_BASE_URL and 'api_base_url' not in kwargs:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.gen_url(url, params, **kwargs)

    @property
    def company_code(self):
        return self._client.company_code



# encoding: utf-8
import time

from . import BaseStorage


class MemoryStorage(BaseStorage):

    def __init__(self):
        self._data = {}

    def get(self, key, default=None):
        ret = self._data.get(key, None)
        if ret is None or len(ret) != 2:
            return default
        else:
            value = ret[0]
            expires_at = ret[1]
            if expires_at is None or expires_at > time.time():
                return value
            else:
                return default

    def set(self, key, value, ttl=None):
        if value is None:
            return
        self._data[key] = (value, int(time.time()) + ttl)

    def delete(self, key):
        self._data.pop(key, None)


# encoding: utf-8

from .base import CecibsBaseAPI


class Auth(CecibsBaseAPI):
    """
    授权接口
    """
    def verify_open(self):
        """
        授权校验接口
        """
        return self._post('/v1/service/verify_open', result_processor=lambda x: True)
